@@ -9,12 +9,14 @@ from inspect import getmembers
 
 import hypothesis
 
+from hypothesis.utils.reflection import convert_keyword_arguments
+
 
 def step(f):
     f.hypothesis_test_step = True
 
     if not hasattr(f, 'hypothesis_test_requirements'):
-        f.hypothesis_test_requirements = ()
+        f.hypothesis_test_requirements = ((), {})
     return f
 
 
@@ -23,16 +25,17 @@ def integrity_test(f):
     return f
 
 
-def requires(*args):
+def requires(*args, **kwargs):
     def alter_function(f):
-        f.hypothesis_test_requirements = args
+        f.hypothesis_test_requirements = (args, kwargs)
         return f
     return alter_function
 
 
 class PreconditionNotMet(Exception):
+
     def __init__(self):
-        Exception.__init__(self, "Precondition not met")
+        Exception.__init__(self, 'Precondition not met')
 
 
 def precondition(t):
@@ -41,21 +44,33 @@ def precondition(t):
 
 
 class TestRun(object):
+
     def __init__(self, cls, steps):
         self.cls = cls
+        if len(steps) >= 1 and steps[0].target.__name__ == '__init__':
+            self.init_args = steps[0].arguments
+            self.init_kwargs = steps[0].kwargs
+            steps = steps[1:]
+        else:
+            self.init_args = ()
+            self.init_kwargs = {}
         self.steps = steps
 
+    def new_value(self):
+        return self.cls(*self.init_args, **self.init_kwargs)
+
     def run(self):
         tests = self.cls.integrity_tests()
-        value = self.cls()
+
+        value = self.new_value()
 
         def run_integrity_tests():
             for t in tests:
                 t(value)
         run_integrity_tests()
-        for step, args in self.steps:
+        for step, args, kwargs in self.steps:
             try:
-                step(value, *args)
+                step(value, *args, **kwargs)
                 run_integrity_tests()
             except PreconditionNotMet:
                 pass
@@ -63,7 +78,7 @@ class TestRun(object):
 
     def prune(self):
         results = []
-        v = self.cls()
+        v = self.new_value()
         for s in self.steps:
             try:
                 s[0](v, *s[1])
@@ -92,13 +107,17 @@ class TestRun(object):
         return len(self.steps)
 
     def __iter__(self):
-        return self.steps.__iter__()
+        if self.init_args or self.init_kwargs:
+            yield Step(self.cls.__init__, self.init_args, self.init_kwargs)
+        for s in self.steps:
+            yield s
 
     def __getitem__(self, i):
         return self.steps[i]
 
 
 class StatefulTest(object):
+
     @classmethod
     def test_steps(cls):
         return cls.functions_with_attributes('hypothesis_test_step')
@@ -114,12 +133,18 @@ class StatefulTest(object):
     @classmethod
     def breaking_example(cls):
         test_run = hypothesis.falsify(TestRun.run, cls)[0]
-        return [(f.__name__,) + args for f, args in test_run]
+        result = []
+        for f, args, kwargs in test_run:
+            args, kwargs = convert_keyword_arguments(f, (None,) + args, kwargs)
+            args = args[1:]
+            result.append((f.__name__,) + args)
+        return result
 
-Step = namedtuple("Step", ("target", "arguments"))
+Step = namedtuple('Step', ('target', 'arguments', 'kwargs'))
 
 
 class StepStrategy(MappedSearchStrategy):
+
     def __init__(self,
                  strategies,
                  descriptor,
@@ -134,16 +159,18 @@ class StepStrategy(MappedSearchStrategy):
             return False
         if x.target != self.descriptor:
             return False
-        return self.mapped_strategy.could_have_produced(x.arguments)
+        return self.mapped_strategy.could_have_produced(
+            (x.arguments, x.kwargs))
 
     def pack(self, x):
-        return Step(self.descriptor, x)
+        return Step(self.descriptor, x[0], x[1])
 
     def unpack(self, x):
-        return x.arguments
+        return (x.arguments, x.kwargs)
 
 
 class StatefulStrategy(MappedSearchStrategy):
+
     def __init__(self,
                  strategies,
                  descriptor,
@@ -153,18 +180,40 @@ class StatefulStrategy(MappedSearchStrategy):
             StepStrategy(strategies, s)
             for s in descriptor.test_steps()
         ]
+        try:
+            init_requirements = (
+                descriptor.__init__.hypothesis_test_requirements)
+        except AttributeError:
+            init_requirements = None
+        self.requires_init = init_requirements is not None
         child_mapper = strategies.new_child_mapper()
         child_mapper.define_specification_for(
             Step,
             lambda sgs, _: sgs.strategy(one_of(step_strategies))
         )
-        self.mapped_strategy = child_mapper.strategy([Step])
+        if self.requires_init:
+            self.mapped_strategy = child_mapper.strategy((
+                init_requirements, [Step]))
+        else:
+            self.mapped_strategy = child_mapper.strategy([Step])
 
-    def pack(self, x):
-        return TestRun(self.descriptor, x)
+    def pack(self, steps):
+        if self.requires_init:
+            (init_args, init_kwargs), steps = steps
+            steps = (
+                [Step(self.descriptor.__init__, init_args, init_kwargs)] +
+                steps
+            )
+        return TestRun(self.descriptor, steps)
 
     def unpack(self, x):
-        return x.steps
+        steps = x.steps
+        if self.requires_init:
+            init = steps[0]
+            steps = steps[1:]
+            return ((init.arguments, init.kwargs), steps)
+        else:
+            return steps
 
     def simplify(self, x):
         pruned = x.prune()
